import semver from 'semver';
import createDebugMessages from 'debug';
import { execa } from 'execa';
const debug = createDebugMessages('@deep-foundation/npm-automation:npm-install');
import { writeFile } from 'fs-extra';
/**
 * Installs a package
 *
 * @throws {@link Error} if the version is invalid
 * @throws {@link Error} if the exit code of npm install is not 0
 *
 * @example
```typescript
await npmInstall({
   packageName: 'typescript',
   version: 'latest',
   deepJsonFilePath,
   packageJsonFilePath,
})
```
 */
export async function npmInstall(param) {
    debug({ param });
    const { name: name, version, deepJsonFilePath, packageJsonFilePath } = param;
    const isVersionValid = version && semver.validRange(version);
    debug({ isVersionValid });
    if (!isVersionValid) {
        throw new Error(`Invalid version ${version}`);
    }
    let npmInstallCommandArgs = [`install`];
    if (version) {
        npmInstallCommandArgs.push(`${name}@${version}`);
    }
    else {
        npmInstallCommandArgs.push(name);
    }
    debug({ npmInstallCommandArgs });
    await execa(`npm`, npmInstallCommandArgs);
    const { default: packageJson } = await import(packageJsonFilePath, { assert: { type: 'json' } });
    debug({ packageJson });
    const packageJsonDependencyVersion = packageJson.dependencies[name];
    debug({ packageJsonDependencyVersion });
    const { default: deepJson } = await import(deepJsonFilePath, { assert: { type: 'json' } });
    debug({ deepJson });
    if (!deepJson.dependencies) {
        deepJson.dependencies = [];
    }
    const deepJsonDependencies = Object.values(deepJson.dependencies);
    const deepJsonDependencyIndex = deepJsonDependencies.findIndex((dependency) => dependency.name === name);
    debug({ deepJsonDependencyIndex });
    if (deepJsonDependencyIndex === -1) {
        deepJsonDependencies.push({
            name,
            version: packageJsonDependencyVersion,
        });
    }
    else {
        deepJsonDependencies[deepJsonDependencyIndex].version =
            packageJsonDependencyVersion;
    }
    deepJson.dependencies = deepJsonDependencies;
    await writeFile(deepJsonFilePath, JSON.stringify(deepJson, null, 2));
}
