import { writeFile } from 'fs/promises';
import semver from 'semver';
import createDebugMessages from 'debug';
/**
 * Syncronizes dependencies between {@link SyncDependenciesParam.deepJsonFilePath} and {@link SyncDependenciesParam.packageJsonFilePath}
 */
export async function syncDependencies(param) {
    const debug = createDebugMessages('@deep-foundation/npm-automation:syncDependencies');
    debug({ param });
    const { deepJsonFilePath, packageJsonFilePath: packageJsonPath, } = param;
    const { default: deepJson } = await import(deepJsonFilePath, { assert: { type: 'json' } });
    debug({ deepJson });
    const { default: packageJson } = await import(packageJsonPath, { assert: { type: 'json' } });
    debug({ packageJson });
    if (!deepJson.dependencies) {
        return;
    }
    if (!packageJson.dependencies) {
        packageJson.dependencies = {};
    }
    const deepJsonDependencies = Object.values(deepJson.dependencies);
    const packageJsonMissingDependenciesFromDeepJson = await getPackageJsonMissingDependenciesFromDeepJson({
        deepJsonDependencies: deepJsonDependencies,
        packageJsonDependencies: packageJson.dependencies
    });
    debug({ packageJsonMissingDependenciesFromDeepJson });
    packageJsonMissingDependenciesFromDeepJson.forEach((dependency) => {
        packageJson.dependencies = { ...packageJson.dependencies, [dependency.name]: `~${dependency.version}` };
    });
    debug({ packageJsonDependenciesAfterAddingMissingDependencies: packageJson.dependencies });
    const syncDependenciesBasedOnDeepJsonResult = await syncDependenciesBasedOnDeepJson({
        deepJsonDependencies: deepJsonDependencies,
        packageJsonDependencies: packageJson.dependencies
    });
    debug({ syncDependenciesBasedOnDeepJsonResult });
    deepJson.dependencies = syncDependenciesBasedOnDeepJsonResult.deepJsonDependencies;
    packageJson.dependencies = syncDependenciesBasedOnDeepJsonResult.packageJsonDependencies;
    debug({ deepJsonDependenciesAfterMergingWithSyncDependenciesBasedOnDeepJsonResult: deepJson.dependencies });
    debug({ packageJsonDependenciesAfterMergingWithSyncDependenciesBasedOnDeepJsonResult: packageJson.dependencies });
    const syncDependenciesBasedOnPackageJsonResult = await syncDependenciesBasedOnPackageJson({
        deepJsonDependencies: deepJsonDependencies,
        packageJsonDependencies: packageJson.dependencies
    });
    debug({ syncDependenciesBasedOnPackageJsonResult });
    deepJson.dependencies = syncDependenciesBasedOnPackageJsonResult.deepJsonDependencies;
    packageJson.dependencies = syncDependenciesBasedOnPackageJsonResult.packageJsonDependencies;
    debug({ deepJsonDependenciesAfterMergingWithSyncDependenciesBasedOnPackageJsonResult: deepJson.dependencies });
    debug({ packageJsonDependenciesAfterMergingWithSyncDependenciesBasedOnPackageJsonResult: packageJson.dependencies });
    debug({ resultDeepJsonDependencies: deepJson.dependencies });
    debug({ resultPackageJsonDependencies: packageJson.dependencies });
    await writeFile(deepJsonFilePath, JSON.stringify(deepJson, null, 2));
    await writeFile(packageJsonPath, JSON.stringify(packageJson, null, 2));
}
async function syncDependenciesBasedOnDeepJson(param) {
    const debug = createDebugMessages('@deep-foundation/npm-automation:syncDependenciesBasedOnDeepJson');
    debug({ param });
    const { deepJsonDependencies, packageJsonDependencies } = param;
    let resultPackageJsonDependencies = { ...packageJsonDependencies };
    debug({ packageJsonDependencies: resultPackageJsonDependencies });
    let resultDeepJsonDependencies = [...deepJsonDependencies];
    debug({ deepJsonDependencies: resultDeepJsonDependencies });
    deepJsonDependencies.forEach((dependency, index) => {
        debug({ dependency, index });
        if (!dependency.version) {
            return;
        }
        const deepJsonDependencyVersionWithoutRange = semver.minVersion(dependency.version)?.version;
        debug({ dependencyVersionWithoutRange: deepJsonDependencyVersionWithoutRange });
        if (!deepJsonDependencyVersionWithoutRange) {
            return;
        }
        ;
        const packageJsonDependencyVersionWithoutRange = semver.minVersion(packageJsonDependencies[dependency.name])?.version;
        debug({ packageJsonDependencyVersionWithoutRange });
        if (!packageJsonDependencyVersionWithoutRange) {
            return;
        }
        ;
        const isDeepJsonVersionGreater = semver.gt(deepJsonDependencyVersionWithoutRange, packageJsonDependencyVersionWithoutRange);
        if (isDeepJsonVersionGreater) {
            resultPackageJsonDependencies[dependency.name] = `~${deepJsonDependencyVersionWithoutRange}`;
        }
        else {
            resultDeepJsonDependencies[index] = {
                name: dependency.name,
                version: deepJsonDependencyVersionWithoutRange
            };
        }
    });
    const result = { packageJsonDependencies: resultPackageJsonDependencies, deepJsonDependencies: resultDeepJsonDependencies };
    debug({ result });
    return result;
}
async function syncDependenciesBasedOnPackageJson(param) {
    const debug = createDebugMessages('@deep-foundation/npm-automation:syncDependenciesBasedOnPackageJson');
    debug({ param });
    const { deepJsonDependencies, packageJsonDependencies } = param;
    const resultPackageJsonDependencies = { ...packageJsonDependencies };
    let resultDeepJsonDependencies = [...deepJsonDependencies];
    debug({ deepJsonDependencies: resultDeepJsonDependencies });
    Object.entries(resultPackageJsonDependencies).forEach(([dependencyName, dependencyVersion]) => {
        debug({ dependencyName, dependencyVersion });
        const packageJsonDependencyVersionWithoutRange = semver.minVersion(dependencyVersion)?.version;
        debug({ dependencyVersionWithoutRange: packageJsonDependencyVersionWithoutRange });
        if (!packageJsonDependencyVersionWithoutRange) {
            return;
        }
        ;
        const deepJsonDependencyIndex = resultDeepJsonDependencies.findIndex(dependency => dependency.name === dependencyName);
        debug({ deepJsonDependencyIndex });
        if (deepJsonDependencyIndex === -1)
            return;
        const deepJsonDependency = resultDeepJsonDependencies[deepJsonDependencyIndex];
        debug({ deepJsonDependency });
        if (!deepJsonDependency.version) {
            return;
        }
        const deepJsonDependencyVersionWithoutRange = semver.minVersion(deepJsonDependency.version)?.version;
        debug({ deepJsonDependencyVersionWithoutRange });
        if (!deepJsonDependencyVersionWithoutRange) {
            return;
        }
        ;
        const isPackageJsonVersionGreater = semver.gt(packageJsonDependencyVersionWithoutRange, deepJsonDependencyVersionWithoutRange);
        if (isPackageJsonVersionGreater) {
            resultDeepJsonDependencies[deepJsonDependencyIndex] = {
                name: dependencyName,
                version: packageJsonDependencyVersionWithoutRange
            };
        }
        else {
            resultPackageJsonDependencies[dependencyName] = `~${deepJsonDependencyVersionWithoutRange}`;
        }
    });
    const result = { packageJsonDependencies: resultPackageJsonDependencies, deepJsonDependencies: resultDeepJsonDependencies };
    debug({ result });
    return result;
}
async function getPackageJsonMissingDependenciesFromDeepJson(param) {
    const debug = createDebugMessages('@deep-foundation/npm-automation:getPackageJsonMissingDependenciesFromDeepJson');
    const { deepJsonDependencies, packageJsonDependencies } = param;
    const missingDependenciesFromDeepJson = deepJsonDependencies.filter((dependency) => packageJsonDependencies[dependency.name] === undefined);
    debug({ missingDependenciesFromDeepJson });
    return missingDependenciesFromDeepJson;
}
